@@ -16,13 +16,15 @@
 """Rebase."""
 
 from bzrlib.config import Config
-from bzrlib.errors import (BzrError, NoSuchFile, UnknownFormatError, 
-                           NoCommonAncestor, UnrelatedBranches)
+from bzrlib.errors import (BzrError, NoSuchFile, UnknownFormatError,
+                           NoCommonAncestor, NoSuchId, UnrelatedBranches)
 from bzrlib.generate_ids import gen_revision_id
+from bzrlib.graph import FrozenHeadsCache
 from bzrlib.merge import Merger
 from bzrlib import osutils
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
+from bzrlib.tsort import topo_sort
 import bzrlib.ui as ui
 
 from maptree import MapTree, map_file_ids
@@ -40,7 +42,7 @@ def rebase_plan_exists(wt):
     :return: boolean
     """
     try:
-        return wt._control_files.get(REBASE_PLAN_FILENAME).read() != ''
+        return wt._transport.get_bytes(REBASE_PLAN_FILENAME) != ''
     except NoSuchFile:
         return False
 
@@ -51,7 +53,7 @@ def read_rebase_plan(wt):
     :param wt: Working Tree for which to write the plan.
     :return: Tuple with last revision info and replace map.
     """
-    text = wt._control_files.get(REBASE_PLAN_FILENAME).read()
+    text = wt._transport.get_bytes(REBASE_PLAN_FILENAME)
     if text == '':
         raise NoSuchFile(REBASE_PLAN_FILENAME)
     return unmarshall_rebase_plan(text)
@@ -63,8 +65,9 @@ def write_rebase_plan(wt, replace_map):
     :param wt: Working Tree for which to write the plan.
     :param replace_map: Replace map (old revid -> (new revid, new parents))
     """
-    wt._control_files.put_utf8(REBASE_PLAN_FILENAME, 
-            marshall_rebase_plan(wt.branch.last_revision_info(), replace_map))
+    content = marshall_rebase_plan(wt.branch.last_revision_info(), replace_map)
+    assert type(content) == str
+    wt._transport.put_bytes(REBASE_PLAN_FILENAME, content)
 
 
 def remove_rebase_plan(wt):
@@ -72,7 +75,7 @@ def remove_rebase_plan(wt):
 
     :param wt: Working Tree for which to remove the plan.
     """
-    wt._control_files.put_utf8(REBASE_PLAN_FILENAME, '')
+    wt._transport.put_bytes(REBASE_PLAN_FILENAME, '')
 
 
 def marshall_rebase_plan(last_rev_info, replace_map):
@@ -110,7 +113,7 @@ def unmarshall_rebase_plan(text):
             # Skip empty lines
             continue
         pts = l.split(" ")
-        replace_map[pts[0]] = (pts[1], pts[2:])
+        replace_map[pts[0]] = (pts[1], tuple(pts[2:]))
     return (last_revision_info, replace_map)
 
 
@@ -124,78 +127,107 @@ def regenerate_default_revid(repository, revid):
     return gen_revision_id(rev.committer, rev.timestamp)
 
 
-def generate_simple_plan(history, start_revid, stop_revid, onto_revid, 
-                         onto_ancestry, get_parents, generate_revid,
-                         skip_full_merged=False):
+def generate_simple_plan(todo_set, start_revid, stop_revid, onto_revid, graph,
+    generate_revid, skip_full_merged=False):
     """Create a simple rebase plan that replays history based 
     on one revision being replayed on top of another.
 
-    :param history: Revision history
+    :param todo_set: A set of revisions to rebase. Only the revisions from
+        stop_revid back through the left hand ancestry are rebased; other
+        revisions are ignored (and references to them are preserved).
     :param start_revid: Id of revision at which to start replaying
     :param stop_revid: Id of revision until which to stop replaying
     :param onto_revid: Id of revision on top of which to replay
-    :param onto_ancestry: Ancestry of onto_revid
-    :param get_parents: Function for obtaining the parents of a revision
+    :param graph: Graph object
     :param generate_revid: Function for generating new revision ids
     :param skip_full_merged: Skip revisions that merge already merged 
                              revisions.
 
     :return: replace map
     """
-    assert start_revid is None or start_revid in history
-    assert stop_revid is None or stop_revid in history
+    assert start_revid is None or start_revid in todo_set, \
+        "invalid start revid(%r), todo_set(%r)" % (start_revid, todo_set)
+    assert stop_revid is None or stop_revid in todo_set, "invalid stop_revid"
     replace_map = {}
-    if start_revid is not None:
-        start_revno = history.index(start_revid)
-    else:
-        start_revno = None
-    if stop_revid is not None:
-        stop_revno = history.index(stop_revid)+1
-    else:
-        stop_revno = None
+    parent_map = graph.get_parent_map(todo_set)
+    order = topo_sort(parent_map)
+    left_most_path = []
+    if stop_revid is None:
+        stop_revid = order[-1]
+    rev = stop_revid
+    while rev in parent_map:
+        left_most_path.append(rev)
+        if rev == start_revid:
+            # manual specified early-stop
+            break
+        rev = parent_map[rev][0]
+    left_most_path.reverse()
+    if start_revid is None:
+        # We need a common base.
+        lca = graph.find_lca(stop_revid, onto_revid)
+        if lca == set([NULL_REVISION]):
+            raise UnrelatedBranches()
     new_parent = onto_revid
-    for oldrevid in history[start_revno:stop_revno]: 
-        oldparents = list(get_parents(oldrevid))
-        assert isinstance(oldparents, list)
-        assert oldparents == [] or \
-                oldparents[0] == history[history.index(oldrevid)-1]
+    todo = left_most_path
+    heads_cache = FrozenHeadsCache(graph)
+    # XXX: The output replacemap'd parents should get looked up in some manner
+    # by the heads cache? RBC 20080719
+    for oldrevid in todo:
+        oldparents = parent_map[oldrevid]
+        assert isinstance(oldparents, tuple), "not tuple: %r" % oldparents
         if len(oldparents) > 1:
-            parents = [new_parent] + filter(lambda p: p not in onto_ancestry or p == onto_revid, oldparents[1:]) 
+            additional_parents = heads_cache.heads(oldparents[1:])
+            parents = [new_parent]
+            for parent in parents:
+                if parent in additional_parents and parent not in parents:
+                    # Use as a parent
+                    parent = replace_map.get(parent, (parent,))[0]
+                    parents.append(parent)
+            parents = tuple(parents)
             if len(parents) == 1 and skip_full_merged:
                 continue
         else:
-            parents = [new_parent]
+            parents = (new_parent,)
         newrevid = generate_revid(oldrevid)
-        assert newrevid != oldrevid
+        assert newrevid != oldrevid, "old and newrevid equal (%r)" % newrevid
+        assert isinstance(parents, tuple), "parents not tuple: %r" % parents
         replace_map[oldrevid] = (newrevid, parents)
         new_parent = newrevid
     return replace_map
 
 
-def generate_transpose_plan(graph, renames, get_parents, generate_revid):
+def generate_transpose_plan(ancestry, renames, graph, generate_revid):
     """Create a rebase plan that replaces a bunch of revisions
     in a revision graph.
 
-    :param graph: Revision graph in which to operate
+    :param ancestry: Ancestry to consider
     :param renames: Renames of revision
-    :param get_parents: Function for determining parents
+    :param graph: Graph object
     :param generate_revid: Function for creating new revision ids
     """
     replace_map = {}
     todo = []
     children = {}
-    for r in graph:
+    parent_map = {}
+    for r, ps in ancestry:
         if not children.has_key(r):
             children[r] = []
-        for p in graph[r]:
+        if ps is None: # Ghost
+            continue
+        parent_map[r] = ps
+        if not children.has_key(r):
+            children[r] = []
+        for p in ps:
             if not children.has_key(p):
                 children[p] = []
             children[p].append(r)
 
+    parent_map.update(graph.get_parent_map(filter(lambda x: not x in parent_map, renames.values())))
+
     # todo contains a list of revisions that need to 
     # be rewritten
-    for r in renames:
-        replace_map[r] = (renames[r], get_parents(renames[r]))
+    for r, v in renames.items():
+        replace_map[r] = (v, parent_map[v])
         todo.append(r)
 
     total = len(todo)
@@ -205,6 +237,7 @@ def generate_transpose_plan(graph, renames, get_parents, generate_revid):
     try:
         while len(todo) > 0:
             r = todo.pop()
+            processed.add(r)
             i += 1
             pb.update('determining dependencies', i, total)
             # Add entry for them in replace_map
@@ -212,19 +245,21 @@ def generate_transpose_plan(graph, renames, get_parents, generate_revid):
                 if c in renames:
                     continue
                 if replace_map.has_key(c):
-                    parents = list(replace_map[c][1])
+                    parents = replace_map[c][1]
                 else:
-                    parents = list(graph[c])
-                assert isinstance(parents, list), \
-                        "Expected list of parents, got: %r" % parents
+                    parents = parent_map[c]
+                assert isinstance(parents, tuple), \
+                        "Expected tuple of parents, got: %r" % parents
                 # replace r in parents with replace_map[r][0]
                 if not replace_map[r][0] in parents:
+                    parents = list(parents)
                     parents[parents.index(r)] = replace_map[r][0]
-                replace_map[c] = (generate_revid(c), parents)
-                assert replace_map[c][0] != c
-            processed.add(r)
-            # Add them to todo[]
-            todo.extend(filter(lambda x: not x in processed, children[r]))
+                    parents = tuple(parents)
+                replace_map[c] = (generate_revid(c), tuple(parents))
+                if replace_map[c][0] == c:
+                    del replace_map[c]
+                elif c not in processed:
+                    todo.append(c)
     finally:
         pb.finished()
 
@@ -242,8 +277,9 @@ def rebase_todo(repository, replace_map):
     :param repository: Repository that contains the revisions
     :param replace_map: Replace map
     """
-    for revid in replace_map:
-        if not repository.has_revision(replace_map[revid][0]):
+    for revid, parent_ids in replace_map.items():
+        assert isinstance(parent_ids, tuple), "replace map parents not tuple"
+        if not repository.has_revision(parent_ids[0]):
             yield revid
 
 
@@ -254,51 +290,24 @@ def rebase(repository, replace_map, replay_fn):
     :param replace_map: Dictionary with revisions to (optionally) rewrite
     :param merge_fn: Function for replaying a revision
     """
-    todo = list(rebase_todo(repository, replace_map))
-    dependencies = {}
-
     # Figure out the dependencies
-    for revid in todo:
-        for p in replace_map[revid][1]:
-            if repository.has_revision(p):
-                continue
-            if not dependencies.has_key(p):
-                dependencies[p] = []
-            dependencies[p].append(revid)
-
+    graph = repository.get_graph()
+    todo = list(graph.iter_topo_order(replace_map.keys()))
     pb = ui.ui_factory.nested_progress_bar()
-    total = len(todo)
-    i = 0
     try:
-        while len(todo) > 0:
-            pb.update('rebase revisions', i, total)
-            i += 1
-            revid = todo.pop()
+        for i, revid in enumerate(todo):
+            pb.update('rebase revisions', i, len(todo))
             (newrevid, newparents) = replace_map[revid]
-            if filter(repository.has_revision, newparents) != newparents:
-                # Not all parents present yet, avoid for now
-                continue
+            assert isinstance(newparents, tuple), "Expected tuple for %r" % newparents
             if repository.has_revision(newrevid):
                 # Was already converted, no need to worry about it again
                 continue
             replay_fn(repository, revid, newrevid, newparents)
-            assert repository.has_revision(newrevid)
-            assert list(repository.revision_parents(newrevid)) == newparents, \
-                   "expected parents %r, got %r" % (newparents, 
-                           repository.revision_parents(newrevid))
-            if dependencies.has_key(newrevid):
-                todo.extend(dependencies[newrevid])
-                del dependencies[newrevid]
     finally:
         pb.finished()
         
-    #assert all(map(repository.has_revision, 
-    #           [replace_map[r][0] for r in replace_map]))
 
-
-
-def replay_snapshot(repository, oldrevid, newrevid, new_parents, 
-                    revid_renames, fix_revid=None):
+def replay_snapshot(repository, oldrevid, newrevid, new_parents):
     """Replay a commit by simply commiting the same snapshot with different 
     parents.
 
@@ -306,9 +315,8 @@ def replay_snapshot(repository, oldrevid, newrevid, new_parents,
     :param oldrevid: Revision id of the revision to copy.
     :param newrevid: Revision id of the revision to create.
     :param new_parents: Revision ids of the new parent revisions.
-    :param revid_renames: Revision id renames for texts.
     """
-    assert isinstance(new_parents, list)
+    assert isinstance(new_parents, tuple), "replay_snapshot: Expected tuple for %r" % new_parents
     mutter('creating copy %r of %r with new parents %r' % 
                                (newrevid, oldrevid, new_parents))
     oldrev = repository.get_revision(oldrevid)
@@ -327,39 +335,46 @@ def replay_snapshot(repository, oldrevid, newrevid, new_parents,
     try:
         # Check what new_ie.file_id should be
         # use old and new parent inventories to generate new_id map
-        fileid_map = map_file_ids(repository, oldrev.parent_ids, new_parents)
-        oldtree = MapTree(repository.revision_tree(oldrevid), fileid_map)
-        total = len(oldtree.inventory)
+        nonghost_oldparents = tuple([p for p in oldrev.parent_ids if repository.has_revision(p)])
+        nonghost_newparents = tuple([p for p in new_parents if repository.has_revision(p)])
+        fileid_map = map_file_ids(repository, nonghost_oldparents, nonghost_newparents)
+        oldtree = repository.revision_tree(oldrevid)
+        mappedtree = MapTree(oldtree, fileid_map)
         pb = ui.ui_factory.nested_progress_bar()
-        i = 0
         try:
-            parent_invs = map(repository.get_revision_inventory, new_parents)
-            transact = repository.get_transaction()
-            for path, ie in oldtree.inventory.iter_entries():
-                pb.update('upgrading file', i, total)
-                ie = ie.copy()
+            old_parent_invs = list(repository.iter_inventories(nonghost_oldparents))
+            new_parent_invs = list(repository.iter_inventories(nonghost_newparents))
+            for i, (path, old_ie) in enumerate(mappedtree.inventory.iter_entries()):
+                pb.update('upgrading file', i, len(mappedtree.inventory))
+                ie = old_ie.copy()
                 # Either this file was modified last in this revision, 
                 # in which case it has to be rewritten
-                if fix_revid is not None:
-                    ie.revision = fix_revid(ie.revision)
-                if ie.revision == oldrevid:
-                    if repository.weave_store.get_weave_or_empty(ie.file_id, 
-                            repository.get_transaction()).has_version(newrevid):
+                if old_ie.revision == oldrevid:
+                    if repository.texts.has_key((ie.file_id, newrevid)):
+                        # Use the existing text
                         ie.revision = newrevid
                     else:
+                        # Create a new text
                         ie.revision = None
                 else:
                     # or it was already there before the commit, in 
                     # which case the right revision should be used
-                    if revid_renames.has_key(ie.revision):
-                        ie.revision = revid_renames[ie.revision]
-                    # make sure at least one of the new parents contains 
-                    # the ie.file_id, ie.revision combination
-                    if len(filter(lambda inv: ie.file_id in inv and inv[ie.file_id].revision == ie.revision, parent_invs)) == 0:
-                        raise ReplayParentsInconsistent(ie.file_id, ie.revision)
-                i += 1
-                builder.record_entry_contents(ie, parent_invs, path, oldtree,
-                        oldtree.path_content_summary(path))
+                    # one of the old parents had this revision, so find that
+                    # and then use the matching new parent
+                    old_file_id = oldtree.inventory.path2id(path)
+                    assert old_file_id is not None
+                    ie = None
+                    for (old_pinv, new_pinv) in zip(old_parent_invs, new_parent_invs):
+                        if (old_pinv.has_id(old_file_id) and
+                            old_pinv[old_file_id].revision == old_ie.revision):
+                            try:
+                                ie = new_pinv[old_ie.file_id].copy()
+                            except NoSuchId:
+                                raise ReplayParentsInconsistent(old_ie.file_id, old_ie.revision)
+                            break
+                    assert ie is not None
+                builder.record_entry_contents(ie, new_parent_invs, path, mappedtree,
+                        mappedtree.path_content_summary(path))
         finally:
             pb.finished()
         builder.finish_inventory()
@@ -375,15 +390,29 @@ def commit_rebase(wt, oldrev, newrevid):
     :param wt: Mutable tree with the changes.
     :param oldrev: Revision info of new revision to commit.
     :param newrevid: New revision id."""
-    assert oldrev.revision_id != newrevid
+    assert oldrev.revision_id != newrevid, "Invalid revid %r" % newrevid
     revprops = dict(oldrev.properties)
     revprops[REVPROP_REBASE_OF] = oldrev.revision_id
+    committer = wt.branch.get_config().username()
+    author = oldrev.get_apparent_author()
+    if author == committer or 'author' in revprops:
+        author = None
     wt.commit(message=oldrev.message, timestamp=oldrev.timestamp, 
-              timezone=oldrev.timezone, revprops=revprops, rev_id=newrevid)
+              timezone=oldrev.timezone, revprops=revprops, rev_id=newrevid,
+              committer=committer, author=author)
     write_active_rebase_revid(wt, None)
 
 
 def replay_determine_base(graph, oldrevid, oldparents, newrevid, newparents):
+    """Determine the base for replaying a revision using merge.
+
+    :param graph: Revision graph.
+    :param oldrevid: Revid of old revision.
+    :param oldparents: List of old parents revids.
+    :param newrevid: Revid of new revision.
+    :param newparents: List of new parents revids.
+    :return: Revision id of the new new revision.
+    """
     # If this was the first commit, no base is needed
     if len(oldparents) == 0:
         return NULL_REVISION
@@ -423,19 +452,17 @@ def replay_delta_workingtree(wt, oldrevid, newrevid, newparents,
     oldrev = wt.branch.repository.get_revision(oldrevid)
     # Make sure there are no conflicts or pending merges/changes 
     # in the working tree
-    if wt.changes_from(wt.basis_tree()).has_changed():
-        raise BzrError("Working tree has uncommitted changes.")
     complete_revert(wt, [newparents[0]])
-    assert not wt.changes_from(wt.basis_tree()).has_changed()
+    assert not wt.changes_from(wt.basis_tree()).has_changed(), "Changes in rev"
 
     oldtree = repository.revision_tree(oldrevid)
     write_active_rebase_revid(wt, oldrevid)
     merger = Merger(wt.branch, this_tree=wt)
     merger.set_other_revision(oldrevid, wt.branch)
-    base_revid = replay_determine_base(repository.get_graph(), 
+    base_revid = replay_determine_base(repository.get_graph(),
                                        oldrevid, oldrev.parent_ids,
                                        newrevid, newparents)
-    mutter('replaying %r as %r with base %r and new parents %r' % 
+    mutter('replaying %r as %r with base %r and new parents %r' %
            (oldrevid, newrevid, base_revid, newparents))
     merger.set_base_revision(base_revid, wt.branch)
     merger.merge_type = merge_type
@@ -452,7 +479,7 @@ def workingtree_replay(wt, map_ids=False, merge_type=None):
     :param map_ids: Whether to try to map between file ids (False for path-based merge)
     """
     def replay(repository, oldrevid, newrevid, newparents):
-        assert wt.branch.repository == repository
+        assert wt.branch.repository == repository, "Different repository"
         return replay_delta_workingtree(wt, oldrevid, newrevid, newparents, 
                                         merge_type=merge_type)
     return replay
@@ -466,7 +493,8 @@ def write_active_rebase_revid(wt, revid):
     """
     if revid is None:
         revid = NULL_REVISION
-    wt._control_files.put_utf8(REBASE_CURRENT_REVID_FILENAME, revid)
+    assert type(revid) == str
+    wt._transport.put_bytes(REBASE_CURRENT_REVID_FILENAME, revid)
 
 
 def read_active_rebase_revid(wt):
@@ -476,7 +504,7 @@ def read_active_rebase_revid(wt):
     :return: Id of the revision that is being rebased.
     """
     try:
-        text = wt._control_files.get(REBASE_CURRENT_REVID_FILENAME).read().rstrip("\n")
+        text = wt._transport.get_bytes(REBASE_CURRENT_REVID_FILENAME).rstrip("\n")
         if text == NULL_REVISION:
             return None
         return text
@@ -503,11 +531,12 @@ def complete_revert(wt, newparents):
             else:
                 os.unlink(abs_path)
     wt.revert(None, old_tree=newtree, backups=False)
-    assert not wt.changes_from(wt.basis_tree()).has_changed()
+    assert not wt.changes_from(wt.basis_tree()).has_changed(), "Rev changed"
     wt.set_parent_ids(newparents)
 
 
 class ReplaySnapshotError(BzrError):
+    """Raised when replaying a snapshot failed."""
     _fmt = """Replaying the snapshot failed: %(message)s."""
 
     def __init__(self, message):
@@ -516,6 +545,7 @@ class ReplaySnapshotError(BzrError):
 
 
 class ReplayParentsInconsistent(BzrError):
+    """Raised when parents were inconsistent."""
     _fmt = """Parents were inconsistent while replaying commit for file id %(fileid)s, revision %(revid)s."""
 
     def __init__(self, fileid, revid):
