@@ -17,7 +17,7 @@
 
 from bzrlib.conflicts import ConflictList
 from bzrlib.errors import UnknownFormatError, NoSuchFile, ConflictsInTree
-from bzrlib.graph import Graph
+from bzrlib.graph import Graph, DictParentsProvider
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport
 from bzrlib.trace import mutter
@@ -41,11 +41,11 @@ class RebasePlanReadWriterTests(TestCase):
 1 bla
 oldrev newrev newparent1 newparent2
 """, marshall_rebase_plan((1, "bla"), 
-                          {"oldrev": ("newrev", ["newparent1", "newparent2"])}))
+                          {"oldrev": ("newrev", ("newparent1", "newparent2"))}))
 
     def test_simple_unmarshall_rebase_plan(self):
         self.assertEquals(((1, "bla"), 
-                          {"oldrev": ("newrev", ["newparent1", "newparent2"])}),
+                          {"oldrev": ("newrev", ("newparent1", "newparent2"))}),
                          unmarshall_rebase_plan("""# Bazaar rebase plan 1
 1 bla
 oldrev newrev newparent1 newparent2
@@ -75,17 +75,18 @@ class ConversionTests(TestCaseWithTransport):
 
         wt.branch.repository.lock_write()
         newrev = replay_snapshot(wt.branch.repository, "bla2", "bla4", 
-                ["bloe"], {"bla": "bloe"})
+                ("bloe",))
         self.assertEqual("bla4", newrev)
         self.assertTrue(wt.branch.repository.has_revision(newrev))
-        self.assertEqual(["bloe"], 
-                wt.branch.repository.revision_parents(newrev))
+        self.assertEqual(("bloe",), 
+                wt.branch.repository.get_parent_map([newrev])[newrev])
         self.assertEqual("bla2", 
             wt.branch.repository.get_revision(newrev).properties["rebase-of"])
         wt.branch.repository.unlock()
 
 
 class PlanCreatorTests(TestCaseWithTransport):
+
     def test_simple_plan_creator(self):
         wt = self.make_branch_and_tree('.')
         b = wt.branch
@@ -99,12 +100,13 @@ class PlanCreatorTests(TestCaseWithTransport):
         file('hello', 'w').write('world')
         wt.commit(message='change hello', rev_id="bla2")
 
-        self.assertEquals({'bla2': ('newbla2', ["bloe"])}, 
-                generate_simple_plan(b.revision_history(), "bla2", None, 
-                    "bloe", 
-                    ["bloe", "bla"],
-                    b.repository.revision_parents, 
-                    lambda y: "new"+y))
+        b.repository.lock_read()
+        graph = b.repository.get_graph()
+        self.assertEquals({'bla2': ('newbla2', ("bloe",))},
+            generate_simple_plan(
+                graph.find_difference(b.last_revision(),"bla")[0],
+                "bla2", None, "bloe", graph, lambda y: "new"+y))
+        b.repository.unlock()
      
     def test_simple_plan_creator_extra_history(self):
         wt = self.make_branch_and_tree('.')
@@ -121,12 +123,15 @@ class PlanCreatorTests(TestCaseWithTransport):
         file('hello', 'w').write('universe')
         wt.commit(message='change hello again', rev_id="bla3")
 
-        self.assertEquals({'bla2': ('newbla2', ["bloe"]), 'bla3': ('newbla3', ['newbla2'])}, 
-                generate_simple_plan(b.revision_history(), "bla2", None, "bloe", 
-                    ["bloe", "bla"],
-                    b.repository.revision_parents,
-                    lambda y: "new"+y))
- 
+        b.repository.lock_read()
+        graph = b.repository.get_graph()
+        self.assertEquals(
+            {'bla2': ('newbla2', ("bloe",)), 'bla3': ('newbla3', ('newbla2',))},
+            generate_simple_plan(
+                graph.find_difference(b.last_revision(),"bloe")[0],
+                "bla2", None, "bloe",
+                graph, lambda y: "new"+y))
+        b.repository.unlock()
 
     def test_generate_transpose_plan(self):
         wt = self.make_branch_and_tree('.')
@@ -151,23 +156,27 @@ class PlanCreatorTests(TestCaseWithTransport):
         wt.add('hello')
         wt.commit(message='add hello', rev_id="lala")
 
+        b.repository.lock_read()
+        graph = b.repository.get_graph()
         self.assertEquals({
-            'blie': ('newblie', ['lala'])},
-            generate_transpose_plan(b.repository.get_revision_graph("blie"), 
-            {"bla": "lala"}, b.repository.revision_parents, lambda y: "new"+y))
+            'blie': ('newblie', ('lala',))},
+            generate_transpose_plan(graph.iter_ancestry(["blie"]), 
+            {"bla": "lala"}, graph, lambda y: "new"+y))
         self.assertEquals({
-            'bla2': ('newbla2', ['lala']),
-            'bla3': ('newbla3', ['newbla2']),
-            'blie': ('newblie', ['lala']),
-            'bloe': ('newbloe', ['lala'])},
-            generate_transpose_plan(b.repository.get_revision_graph(), 
+            'bla2': ('newbla2', ('lala',)),
+            'bla3': ('newbla3', ('newbla2',)),
+            'blie': ('newblie', ('lala',)),
+            'bloe': ('newbloe', ('lala',))},
+            generate_transpose_plan(graph.iter_ancestry(b.repository._all_revision_ids()), 
             {"bla": "lala"}, 
-            b.repository.revision_parents, lambda y: "new"+y))
+            graph, lambda y: "new"+y))
+        b.repository.unlock()
 
     def test_generate_transpose_plan_one(self):
-        self.assertEquals({"bla": ("newbla", ["lala"])},
-                generate_transpose_plan({"bla": ["bloe"], "bloe": []},
-                    {"bloe": "lala"}, {}.get, lambda y: "new"+y))
+        graph = Graph(DictParentsProvider({"bla": ("bloe",), "bloe": (), "lala": ()}))
+        self.assertEquals({"bla": ("newbla", ("lala",))},
+                generate_transpose_plan(graph.iter_ancestry(["bla", "bloe"]),
+                    {"bloe": "lala"}, graph, lambda y: "new"+y))
 
     def test_plan_with_already_merged(self):
         """We need to use a merge base that makes sense. 
@@ -188,16 +197,16 @@ class PlanCreatorTests(TestCaseWithTransport):
         E -> (E', [D', C])
         """
         parents_map = {
-                "A": [],
-                "B": ["A"],
-                "C": ["B"],
-                "D": ["A"],
-                "E": ["D", "B"]
+                "A": (),
+                "B": ("A",),
+                "C": ("B",),
+                "D": ("A",),
+                "E": ("D", "B")
         }
-        self.assertEquals({"D": ("D'", ["C"]), "E": ("E'", ["D'"])}, 
-                generate_simple_plan(["A", "D", "E"], 
-                                     "D", None, "C", ["A", "B", "C"], 
-                    parents_map.get, lambda y: y+"'"))
+        graph = Graph(DictParentsProvider(parents_map))
+        self.assertEquals({"D": ("D'", ("C",)), "E": ("E'", ("D'",))},
+            generate_simple_plan(["D", "E"], "D", None, "C",
+                graph, lambda y: y+"'"))
 
     def test_plan_with_already_merged_skip_merges(self):
         """We need to use a merge base that makes sense. 
@@ -217,16 +226,16 @@ class PlanCreatorTests(TestCaseWithTransport):
         D -> (D', [C])
         """
         parents_map = {
-                "A": [],
-                "B": ["A"],
-                "C": ["B"],
-                "D": ["A"],
-                "E": ["D", "B"]
+                "A": (),
+                "B": ("A",),
+                "C": ("B",),
+                "D": ("A",),
+                "E": ("D", "B")
         }
-        self.assertEquals({"D": ("D'", ["C"])}, 
-                generate_simple_plan(["A", "D", "E"], 
-                                     "D", None, "C", ["A", "B", "C"], 
-                    parents_map.get, lambda y: y+"'", True))
+        graph = Graph(DictParentsProvider(parents_map))
+        self.assertEquals({"D": ("D'", ("C",))},
+                generate_simple_plan(["D", "E"], "D", None, "C",
+                    graph, lambda y: y+"'", True))
  
 
 class PlanFileTests(TestCaseWithTransport):
@@ -236,17 +245,17 @@ class PlanFileTests(TestCaseWithTransport):
 
    def test_rebase_plan_exists_empty(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_PLAN_FILENAME, "")
+        wt._transport.put_bytes(REBASE_PLAN_FILENAME, "")
         self.assertFalse(rebase_plan_exists(wt))
 
    def test_rebase_plan_exists(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_PLAN_FILENAME, "foo")
+        wt._transport.put_bytes(REBASE_PLAN_FILENAME, "foo")
         self.assertTrue(rebase_plan_exists(wt))
 
    def test_remove_rebase_plan(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_PLAN_FILENAME, "foo")
+        wt._transport.put_bytes(REBASE_PLAN_FILENAME, "foo")
         remove_rebase_plan(wt)
         self.assertFalse(rebase_plan_exists(wt))
 
@@ -265,7 +274,7 @@ class PlanFileTests(TestCaseWithTransport):
         self.assertEqualDiff("""# Bazaar rebase plan 1
 1 bla
 oldrev newrev newparent1 newparent2
-""", wt._control_files.get(REBASE_PLAN_FILENAME).read())
+""", wt._transport.get_bytes(REBASE_PLAN_FILENAME))
 
    def test_read_rebase_plan_nonexistant(self):
         wt = self.make_branch_and_tree('.')
@@ -273,17 +282,17 @@ oldrev newrev newparent1 newparent2
 
    def test_read_rebase_plan_empty(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_PLAN_FILENAME, "")
+        wt._transport.put_bytes(REBASE_PLAN_FILENAME, "")
         self.assertRaises(NoSuchFile, read_rebase_plan, wt)
         
    def test_read_rebase_plan(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_PLAN_FILENAME, """# Bazaar rebase plan 1
+        wt._transport.put_bytes(REBASE_PLAN_FILENAME, """# Bazaar rebase plan 1
 1 bla
 oldrev newrev newparent1 newparent2
 """)
         self.assertEquals(((1, "bla"), 
-            {"oldrev": ("newrev", ["newparent1", "newparent2"])}),
+            {"oldrev": ("newrev", ("newparent1", "newparent2"))}),
             read_rebase_plan(wt))
 
 
@@ -294,12 +303,12 @@ class CurrentRevidFileTests(TestCaseWithTransport):
 
     def test_read_null(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_CURRENT_REVID_FILENAME, NULL_REVISION)
+        wt._transport.put_bytes(REBASE_CURRENT_REVID_FILENAME, NULL_REVISION)
         self.assertIs(None, read_active_rebase_revid(wt))
 
     def test_read(self):
         wt = self.make_branch_and_tree('.')
-        wt._control_files.put_utf8(REBASE_CURRENT_REVID_FILENAME, "bla")
+        wt._transport.put_bytes(REBASE_CURRENT_REVID_FILENAME, "bla")
         self.assertEquals("bla", read_active_rebase_revid(wt))
 
     def test_write(self):
@@ -343,8 +352,7 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.add(["afile"])
         wt.commit("bla", rev_id="oldcommit")
         wt.branch.repository.lock_write()
-        replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", [],
-                        {})
+        replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", ())
         wt.branch.repository.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
@@ -372,26 +380,25 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.branch.repository.lock_write()
         self.assertRaises(
                 ReplayParentsInconsistent, replay_snapshot, 
-                wt.branch.repository, "oldcommit", "newcommit", 
-                ["base"], {"oldparent": "base"})
+                wt.branch.repository, "oldcommit", "newcommit", ("base",))
         wt.branch.repository.unlock()
 
     def test_two_revisions(self):
         wt = self.make_branch_and_tree("old")
-        self.build_tree(['old/afile', 'old/notherfile'])
+        self.build_tree_contents([('old/afile', 'afilecontents'), ('old/notherfile', 'notherfilecontents')])
         wt.add(["afile"], ["somefileid"])
         wt.commit("bla", rev_id="oldparent")
         wt.add(["notherfile"])
         wt.commit("bla", rev_id="oldcommit")
         oldrepos = wt.branch.repository
         wt = self.make_branch_and_tree("new")
-        self.build_tree(['new/afile', 'new/notherfile'])
+        self.build_tree_contents([('new/afile', 'afilecontents'), ('new/notherfile', 'notherfilecontents')])
         wt.add(["afile"], ["afileid"])
         wt.commit("bla", rev_id="newparent")
         wt.branch.repository.fetch(oldrepos)
         wt.branch.repository.lock_write()
         replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", 
-                ["newparent"], {"oldparent": "newparent"})
+                ("newparent",))
         wt.branch.repository.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
@@ -418,15 +425,15 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.commit("bla", rev_id="newparent")
         wt.branch.repository.fetch(oldrepos)
         wt.branch.repository.lock_write()
-        self.assertRaises(ReplayParentsInconsistent, 
-                          replay_snapshot, wt.branch.repository, 
-                          "oldcommit", "newcommit", 
-                        ["newparent"], revid_renames={})
+        replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", 
+                        ("newparent",))
         wt.branch.repository.unlock()
 
     def test_multi_revisions(self):
         wt = self.make_branch_and_tree("old")
-        self.build_tree(['old/afile', 'old/sfile', 'old/notherfile'])
+        self.build_tree_contents([('old/afile', 'afilecontent'), 
+                         ('old/sfile', 'sfilecontent'),
+                         ('old/notherfile', 'notherfilecontent')])
         wt.add(['sfile'])
         wt.add(["afile"], ["somefileid"])
         wt.commit("bla", rev_id="oldgrandparent")
@@ -436,7 +443,9 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.commit("bla", rev_id="oldcommit")
         oldrepos = wt.branch.repository
         wt = self.make_branch_and_tree("new")
-        self.build_tree(['new/afile', 'new/sfile', 'new/notherfile'])
+        self.build_tree_contents([('new/afile', 'afilecontent'), 
+                         ('new/sfile', 'sfilecontent'),
+                         ('new/notherfile', 'notherfilecontent')])
         wt.add(['sfile'])
         wt.add(["afile"], ["afileid"])
         wt.commit("bla", rev_id="newgrandparent")
@@ -445,8 +454,7 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.branch.repository.fetch(oldrepos)
         wt.branch.repository.lock_write()
         replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", 
-                ["newparent"], {"oldgrandparent": "newgrandparent", 
-                                "oldparent": "newparent"})
+                ("newparent",))
         wt.branch.repository.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
@@ -476,7 +484,7 @@ class ReplaySnapshotTests(TestCaseWithTransport):
         wt.branch.repository.fetch(oldrepos)
         wt.branch.repository.lock_write()
         replay_snapshot(wt.branch.repository, "oldcommit", "newcommit", 
-                ["newparent"], {"oldparent": "newparent"})
+                ("newparent",))
         wt.branch.repository.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
@@ -523,10 +531,8 @@ class TestReplayWorkingtree(TestCaseWithTransport):
         self.build_tree(['new/bfile'])
         wt.add(["bfile"], ids=["newid"])
         wt.commit("bla", rev_id="newparent")
-        wt.lock_write()
         replay_delta_workingtree(wt, "oldcommit", "newcommit", 
             ["newparent"])
-        wt.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
         self.assertEquals(["newparent"], newrev.parent_ids)
@@ -535,24 +541,41 @@ class TestReplayWorkingtree(TestCaseWithTransport):
         self.assertEquals(oldrev.timezone, newrev.timezone)
 
     def test_multiple(self):
+        # rebase from 
+        # base: []
+        # oldparent: [base]
+        # newparent: [base]
+        # oldcommit: [oldparent, ghost]
+        # create newcommit by rebasing oldcommit from oldparent to newparent,
+        # keeping the merge of ghost.
+        # Common base:
         wt = self.make_branch_and_tree("old")
         wt.commit("base", rev_id="base")
-        self.build_tree(['old/afile'])
+        # oldparent:
+        self.build_tree_contents([('old/afile', 'base content')])
         wt.add(["afile"], ids=["originalid"])
         wt.commit("bla", rev_id="oldparent")
+        # oldcommit (the delta getting rebased)
+        #  - change the content of afile to be 'bloe'
         file("old/afile", "w").write("bloe")
         wt.add_pending_merge("ghost")
         wt.commit("bla", rev_id="oldcommit")
-        wt = wt.bzrdir.sprout("new").open_workingtree()
-        self.build_tree(['new/bfile'])
-        wt.add(["bfile"], ids=["newid"])
+        # newparent (the new base for the rebased commit)
+        new_tree = wt.bzrdir.sprout("new",
+            revision_id='base').open_workingtree()
+        new_tree.branch.repository.fetch(wt.branch.repository)
+        wt = new_tree
+        self.build_tree_contents([('new/afile', 'base content')])
+        wt.add(["afile"], ids=["originalid"])
         wt.commit("bla", rev_id="newparent")
+        # And do it!
         wt.lock_write()
-        replay_delta_workingtree(wt, "oldcommit", "newcommit", 
-            ["newparent", "ghost"])
+        replay_delta_workingtree(wt, "oldcommit", "newcommit",
+            ("newparent", "ghost"))
         wt.unlock()
         oldrev = wt.branch.repository.get_revision("oldcommit")
         newrev = wt.branch.repository.get_revision("newcommit")
+        self.assertEquals(["oldparent", "ghost"], oldrev.parent_ids)
         self.assertEquals(["newparent", "ghost"], newrev.parent_ids)
         self.assertEquals("newcommit", newrev.revision_id)
         self.assertEquals(oldrev.timestamp, newrev.timestamp)
@@ -635,6 +658,9 @@ class ReplayDetermineBaseTests(TestCase):
     def setUp(self):
         self.graph = Graph(self)
 
+    def get_parent_map(self, keys):
+        return dict((revid, self._parent_dict[revid]) for k in self._parent_dict if k in keys)
+
     def get_parents(self, revid):
         return self._parent_dict[revid]
 
